deprelja = {
  "nsubj":"主語",
  "nsubj:pass":"主語[受動文]",
  "obj":"目的語",
  "iobj":"間接目的語",
  "obl":"斜格補語",
  "obl:tmod":"斜格補語[時]",
  "obl:lmod":"斜格補語[場所]",
  "vocative":"呼称語",
  "expl":"形式語",
  "dislocated":"外置語",
  "nmod":"体言による連体修飾語",
  "nummod":"数量による修飾語",
  "csubj":"節主語",
  "csubj:pass":"節主語[受動文]",
  "ccomp":"節目的語",
  "xcomp":"節補語",
  "advcl":"連用修飾節",
  "acl":"連体修飾節",
  "advmod":"連用修飾語",
  "discourse":"談話要素",
  "discourse:sp":"談話要素[文助詞]",
  "amod":"用言による連体修飾語",
  "aux":"動詞補助成分",
  "cop":"繫辞",
  "mark":"標識",
  "det":"決定詞",
  "clf":"類別詞",
  "case":"格表示",
  "conj":"接続",
  "cc":"接続詞",
  "fixed":"固着",
  "compound":"複合",
  "flat":"並列",
  "flat:vv":"並列[動詞類]",
  "list":"細目",
  "parataxis":"隣接表現",
  "orphan":"親なし",
  "appos":"同格",
  "punct":"句読点",
  "root":"親"
}
